/* 
 * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 * Copyright (C) 2005/2006, Anthony Minessale II <anthmct@yahoo.com>
 *
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 *
 * The Initial Developer of the Original Code is
 * Anthony Minessale II <anthmct@yahoo.com>
 * Portions created by the Initial Developer are Copyright (C)
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Brian West <brian.west@mac.com>
 * Anthony Minessale II <anthmct@yahoo.com>
 *
 *
 * mod_snmp.c -- SNMP Module
 *
 */
#include <switch.h>

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>

#define FS_PEN 27880

static uint64_t snmp_switch_sessions_count = 0;

const char *app_name = "mod_snmp";

int netsnmp_running = 1;

static const char modname[] = "mod_snmp";

int snmp_watcher_handler(netsnmp_mib_handler * handler,
						 netsnmp_handler_registration * reginfo, netsnmp_agent_request_info * reqinfo, netsnmp_request_info * requests)
{
	netsnmp_request_info *request;
	netsnmp_variable_list *var;
	void *data_context = NULL;
	char *retval;
	size_t retval_len = 0;
	for (request = requests; request; request = request->next) {
		printf("WTF MODE %d\n", reqinfo->mode);
		switch (reqinfo->mode) {
		case MODE_GET:
			printf("WTF GET\n");
			break;
		case MODE_SET_ACTION:
			printf("WTF SET\n");

			break;
		default:
			break;
		}
	}
	snprintf(retval, retval_len, "FUCK YOU");
	return SNMP_ERR_NOERROR;
}



void init_watched_string(void)
{
	static char my_string[256] = "So long, and thanks for all the fish!";
	oid my_registration_oid[] = { 1, 3, 6, 1, 4, 1, FS_PEN, 2, 1, 3, 0 };

	netsnmp_handler_registration *reginfo;
	netsnmp_watcher_info *watcher_info;
	int watcher_flags;

	reginfo = netsnmp_create_handler_registration("my example string", snmp_watcher_handler,
												  my_registration_oid, OID_LENGTH(my_registration_oid), HANDLER_CAN_RWRITE);


	watcher_flags = WATCHER_MAX_SIZE;

	//watcher_info = netsnmp_create_watcher_info(my_string, strlen(my_string), ASN_OCTET_STR, watcher_flags);
	watcher_info = netsnmp_create_watcher_info(NULL, 0, ASN_OCTET_STR, watcher_flags);

	watcher_info->max_size = sizeof(my_string);

	netsnmp_register_watched_instance(reginfo, watcher_info);
}


int snmp_switch_sessions_count_handler(netsnmp_mib_handler * handler,
									   netsnmp_handler_registration * reginfo, netsnmp_agent_request_info * reqinfo, netsnmp_request_info * requests)
{
	netsnmp_request_info *request;
	netsnmp_variable_list *var;
	void *data_context = NULL;
	char *retval;
	size_t retval_len = 0;
	for (request = requests; request; request = request->next) {
		switch (reqinfo->mode) {
		case MODE_GET:
			snmp_switch_sessions_count = switch_core_session_count();
			break;
		default:
			break;
		}
	}

	return SNMP_ERR_NOERROR;
}

void init_snmp_switch_sessions_count(void)
{
	static oid snmp_switch_sessions_count_oid[] = { 1, 3, 6, 1, 4, 1, FS_PEN, 2, 4, 1, 1, 2, 0 };

	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE,
					  "Initalizing snmp_switch_sessions_count scalar integer.  Default value = %d\n", snmp_switch_sessions_count);

	netsnmp_register_read_only_uint_instance("snmp_switch_sessions_count",
											 snmp_switch_sessions_count_oid,
											 OID_LENGTH(snmp_switch_sessions_count_oid), &snmp_switch_sessions_count, snmp_switch_sessions_count_handler);


	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Done initalizing snmp_switch_sessions_count module\n");
}

static switch_loadable_module_interface_t snmp_module_interface = {
	/*.module_name */ modname,
	/*.endpoint_interface */ NULL,
	/*.timer_interface */ NULL,
	/*.dialplan_interface */ NULL,
	/*.codec_interface */ NULL,
	/*.application_interface */ NULL,
	/*.api_interface */ NULL,
	/*.file_interface */ NULL,
	/*.speech_interface */ NULL,
	/*.directory_interface */ NULL
};

SWITCH_MOD_DECLARE(switch_status_t) switch_module_load(const switch_loadable_module_interface_t **module_interface, char *filename)
{
	*module_interface = &snmp_module_interface;

	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "mod_snmp Loaded!\n");

	return SWITCH_STATUS_SUCCESS;
}

SWITCH_MOD_DECLARE(switch_status_t) switch_module_shutdown(void)
{
	int loops = 0;

	netsnmp_running = -1;
	while (netsnmp_running) {
		switch_yield(100000);
		loops++;
		if (loops > 50) {
			break;
		}
	}
	return SWITCH_STATUS_SUCCESS;
}

SWITCH_MOD_DECLARE(switch_status_t) switch_module_runtime(void)
{
	netsnmp_ds_set_boolean(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_ROLE, 1);

	SOCK_STARTUP;
	init_agent(app_name);

	/* This should be set in the config section for this module 
	 * which doesn't exist yet.
	 */

	netsnmp_ds_set_string(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_X_SOCKET, "/tmp/agentx/master");

	init_snmp_switch_sessions_count();
	init_watched_string();
	init_snmp(app_name);
	while (netsnmp_running == 1) {
		agent_check_and_process(1);
	}

	snmp_shutdown(app_name);
	SOCK_CLEANUP;

	netsnmp_running = 0;

	return SWITCH_STATUS_TERM;
}

/* For Emacs:
 * Local Variables:
 * mode:c
 * indent-tabs-mode:nil
 * tab-width:4
 * c-basic-offset:4
 * End:
 * For VIM:
 * vim:set softtabstop=4 shiftwidth=4 tabstop=4 expandtab:
 */
